package com.portlandwebworks.commons.template;

import com.portlandwebworks.commons.domain.Role;
import com.portlandwebworks.commons.domain.User;

import java.net.URL;
import java.util.*;

/**
 * This software is the property of Portland Webworks.
 *
 * This provider will choose a template based on the assigned role(s) of the user provided.  The assumed naming convention
 * for the templates is <code>templateName_roleName.vm</code>.
 *
 * If the user has multiple roles, the first role in the list returned by User#getRoles(), sorted by ordinal value of
 * the role code.  If no matches using the role name suffix is found, the base template path will be used.
 *
 * If reverseRoleOrder is set to true, the user's roles will be checked in reverse order of the ordinal values of the
 * role code.
 *
 * @author Ronnie Northup
 */
public class RoleBasedVelocityTemplateProvider extends VelocityTemplateProvider
{
	private HashMap<String, Boolean> resourceLookups = new HashMap<String, Boolean>();

	protected boolean reverseRoleOrder;

	public void setReverseRoleOrder(boolean reverseRoleOrder)
	{
		this.reverseRoleOrder = reverseRoleOrder;
	}

	@Override
	protected String resolveTemplatePath(String templatePath, User user)
	{
		final boolean reverse = this.reverseRoleOrder;

		if (user != null) {
			List<Role> userRoles = (List<Role>)user.getRoles();
			if (userRoles.size() > 1) {  // Collections.sort() does not check for empty or single value in collection
				Collections.sort(userRoles, new Comparator<Role>() {
					public int compare(Role o1, Role o2) {
						return reverse?
								o2.getCode().compareTo(o1.getCode()):
								o1.getCode().compareTo(o2.getCode());
					}
				});
			}

			for (Role role : userRoles) {
				// Iterate through the user's roles, looking for a template that has the role code appended to the template name.
				String resourcePath = templateDirectory + "/" + templatePath + "_" + role.getCode() + ".vm";
				if (this.findResourceUsingClassLoader(resourcePath)) {
					return resourcePath;
				}
			}
		}

		// Complain if the template path without any role appended is invalid.
		if (! this.findResourceUsingClassLoader(templateDirectory + "/" + templatePath)) {
			throw new RuntimeException("Could not find template " + templatePath + " for any role");
		}

		return templatePath;
	}

	protected boolean findResourceUsingClassLoader(String resourcePath)
	{
		Boolean found = resourceLookups.get(resourcePath);
		if (found == null) {
			found = (null != this.getClass().getResource(resourcePath));
			resourceLookups.put(resourcePath, found);
		}
		return found;
	}
}
